@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@xwax.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,6 +19,8 @@
 
 #include <assert.h>
 
+#include "controller.h"
+#include "cues.h"
 #include "deck.h"
 #include "rig.h"
 
@@ -43,17 +45,19 @@ static const struct record no_record = {
 
 int deck_init(struct deck *deck, struct rt *rt)
 {
-    unsigned int sample_rate;
+    unsigned int rate;
 
     assert(deck->importer != NULL);
 
     if (rt_add_device(rt, &deck->device) == -1)
         return -1;
 
+    deck->ncontrol = 0;
     deck->record = &no_record;
-    sample_rate = device_sample_rate(&deck->device);
-    player_init(&deck->player, sample_rate, track_get_empty(),
-                &deck->timecoder);
+    deck->punch = NO_PUNCH;
+    rate = device_sample_rate(&deck->device);
+    player_init(&deck->player, rate, track_get_empty(), &deck->timecoder);
+    cues_reset(&deck->cues);
 
     /* The timecoder and player are driven by requests from
      * the audio device */
@@ -109,3 +113,67 @@ void deck_clone(struct deck *deck, const struct deck *from)
     deck->record = from->record;
     player_clone(&deck->player, &from->player);
 }
+
+/*
+ * Clear the cue point, ready to be set again
+ */
+
+void deck_unset_cue(struct deck *d, unsigned int label)
+{
+    cues_unset(&d->cues, label);
+}
+
+/*
+ * Seek the current playback position to a cue point position,
+ * or set the cue point if unset
+ */
+
+void deck_cue(struct deck *d, unsigned int label)
+{
+    double p;
+
+    p = cues_get(&d->cues, label);
+    if (p == CUE_UNSET)
+        cues_set(&d->cues, label, player_get_elapsed(&d->player));
+    else
+        player_seek_to(&d->player, p);
+}
+
+/*
+ * Seek to a cue point ready to return from it later. Overrides an
+ * existing punch operation.
+ */
+
+void deck_punch_in(struct deck *d, unsigned int label)
+{
+    double p, e;
+
+    e = player_get_elapsed(&d->player);
+    p = cues_get(&d->cues, label);
+    if (p == CUE_UNSET) {
+        cues_set(&d->cues, label, e);
+        return;
+    }
+
+    if (d->punch != NO_PUNCH)
+        e -= d->punch;
+
+    player_seek_to(&d->player, p);
+    d->punch = p - e;
+}
+
+/*
+ * Return from a cue point
+ */
+
+void deck_punch_out(struct deck *d)
+{
+    double e;
+
+    if (d->punch == NO_PUNCH)
+        return;
+
+    e = player_get_elapsed(&d->player);
+    player_seek_to(&d->player, e - d->punch);
+    d->punch = NO_PUNCH;
+}
