@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@xwax.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -36,6 +36,7 @@
 #include "player.h"
 #include "rig.h"
 #include "selector.h"
+#include "status.h"
 #include "timecoder.h"
 #include "xwax.h"
 
@@ -51,6 +52,10 @@
 
 #define EM_FONT "DejaVuSans-Oblique.ttf"
 
+#define BIG_FONT "DejaVuSans-Bold.ttf"
+#define BIG_FONT_SIZE 14
+#define BIG_FONT_SPACE 19
+
 #define CLOCK_FONT FONT
 #define CLOCK_FONT_SIZE 32
 
@@ -59,11 +64,16 @@
 
 #define DETAIL_FONT "DejaVuSansMono.ttf"
 #define DETAIL_FONT_SIZE 9
+#define DETAIL_FONT_SPACE 12
 
 /* Screen dimensions */
 
+#define DEFAULT_WIDTH 960
+#define DEFAULT_HEIGHT 720
+
 #define BORDER 12
 #define SPACER 8
+#define HALF_SPACER 4
 
 #define CURSOR_WIDTH 4
 
@@ -73,17 +83,19 @@
 #define LIBRARY_MIN_WIDTH 64
 #define LIBRARY_MIN_HEIGHT 64
 
-#define DEFAULT_WIDTH 960
-#define DEFAULT_HEIGHT 720
 #define DEFAULT_METER_SCALE 8
 
 #define MAX_METER_SCALE 11
 
 #define SEARCH_HEIGHT (FONT_SPACE)
-#define STATUS_HEIGHT (FONT_SPACE)
+#define STATUS_HEIGHT (DETAIL_FONT_SPACE)
 
+#define BPM_WIDTH 32
+#define SORT_WIDTH 22
 #define RESULTS_ARTIST_WIDTH 200
 
+#define TOKEN_SPACE 2
+
 #define CLOCKS_WIDTH 160
 
 #define SPINNER_SIZE (CLOCK_FONT_SIZE * 2 - 6)
@@ -99,15 +111,11 @@
 #define FUNC_RECUE 1
 #define FUNC_TIMECODE 2
 
-/* State variables used to trigger certain actions */
-
-#define UPDATE_NONE 0
-#define UPDATE_REDRAW 1
-
 /* Types of SDL_USEREVENT */
 
 #define EVENT_TICKER 0
 #define EVENT_QUIT 1
+#define EVENT_STATUS 2
 
 /* Macro functions */
 
@@ -130,7 +138,8 @@ static const char *font_dirs[] = {
     NULL
 };
 
-static TTF_Font *clock_font, *deci_font, *detail_font, *font, *em_font;
+static TTF_Font *clock_font, *deci_font, *detail_font,
+    *font, *em_font, *big_font;
 
 static SDL_Color background_col = {0, 0, 0, 255},
     text_col = {224, 224, 224, 255},
@@ -140,13 +149,15 @@ static SDL_Color background_col = {0, 0, 0, 255},
     cursor_col = {192, 0, 0, 255},
     selected_col = {0, 48, 64, 255},
     detail_col = {128, 128, 128, 255},
-    needle_col = {255, 255, 255, 255};
+    needle_col = {255, 255, 255, 255},
+    artist_col = {16, 64, 0, 255},
+    bpm_col = {64, 16, 0, 255};
 
 static int spinner_angle[SPINNER_SIZE * SPINNER_SIZE];
 
+static int width = DEFAULT_WIDTH, height = DEFAULT_HEIGHT,
+    meter_scale = DEFAULT_METER_SCALE;
 static pthread_t ph;
-static struct deck *deck;
-static size_t ndeck;
 static struct selector selector;
 
 struct rect {
@@ -228,6 +239,29 @@ static void split_right(const struct rect *source, struct rect *left,
     split_left(source, left, right, source->w - v - space, space);
 }
 
+/*
+ * Shrink a rectangle to leave a small border of standard size
+ */
+
+static void shrink(const struct rect *in, struct rect *out)
+{
+    if (TOKEN_SPACE * 2 >= in->w) {
+        out->x = in->x;
+        out->w = in->w;
+    } else {
+        out->x = in->x + TOKEN_SPACE;
+        out->w = in->w - TOKEN_SPACE * 2;
+    }
+
+    if (TOKEN_SPACE * 2 >= in->h) {
+        out->y = in->y;
+        out->h = in->h;
+    } else {
+        out->y = in->y + TOKEN_SPACE;
+        out->h = in->h - TOKEN_SPACE * 2;
+    }
+}
+
 /*
  * Convert the given time (in milliseconds) to displayable time
  */
@@ -384,6 +418,10 @@ static int load_fonts(void)
     if (!em_font)
         return -1;
 
+    big_font = open_font(BIG_FONT, BIG_FONT_SIZE);
+    if (!big_font)
+        return -1;
+
     detail_font = open_font(DETAIL_FONT, DETAIL_FONT_SIZE);
     if (!detail_font)
         return -1;
@@ -401,6 +439,7 @@ static void clear_fonts(void)
     TTF_CloseFont(deci_font);
     TTF_CloseFont(font);
     TTF_CloseFont(em_font);
+    TTF_CloseFont(big_font);
     TTF_CloseFont(detail_font);
 }
 
@@ -482,6 +521,139 @@ static void draw_rect(SDL_Surface *surface, const struct rect *rect,
     SDL_FillRect(surface, &b, palette(surface, &col));
 }
 
+/*
+ * Draw some text in a box
+ */
+
+static void draw_token(SDL_Surface *surface, const struct rect *rect,
+                       const char *buf,
+                       SDL_Color text_col, SDL_Color col, SDL_Color bg_col)
+{
+    struct rect b;
+
+    draw_rect(surface, rect, bg_col);
+    shrink(rect, &b);
+    draw_text(surface, &b, buf, detail_font, text_col, col);
+}
+
+/*
+ * Dim a colour for display
+ */
+
+static SDL_Color dim(const SDL_Color x, int n)
+{
+    SDL_Color c;
+
+    c.r = x.r >> n;
+    c.g = x.g >> n;
+    c.b = x.b >> n;
+
+    return c;
+}
+
+/*
+ * Get a colour from RGB values
+ */
+
+static SDL_Color rgb(double r, double g, double b)
+{
+    SDL_Color c;
+
+    c.r = r * 255;
+    c.g = g * 255;
+    c.b = b * 255;
+
+    return c;
+}
+
+/*
+ * Get a colour from HSV values
+ *
+ * Pre: h is in degrees, in the range 0.0 to 360.0
+ */
+
+static SDL_Color hsv(double h, double s, double v)
+{
+    int i;
+    double f, p, q, t;
+
+    if (s == 0.0)
+        return rgb(v, v, v);
+
+    h /= 60;
+    i = floor(h);
+    f = h - i;
+    p = v * (1 - s);
+    q = v * (1 - s * f);
+    t = v * (1 - s * (1 - f));
+
+    switch (i) {
+    case 0:
+        return rgb(v, t, p);
+    case 1:
+        return rgb(q, v, p);
+    case 2:
+        return rgb(p, v, t);
+    case 3:
+        return rgb(p, q, v);
+    case 4:
+        return rgb(t, p, v);
+    case 5:
+    case 6:
+        return rgb(v, p, q);
+    default:
+        abort();
+    }
+}
+
+static bool show_bpm(double bpm)
+{
+    return (bpm > 20.0 && bpm < 400.0);
+}
+
+/*
+ * Draw the beats-per-minute indicator
+ */
+
+static void draw_bpm(SDL_Surface *surface, const struct rect *rect, double bpm,
+                     SDL_Color bg_col)
+{
+    static const double min = 60.0, max = 240.0;
+    char buf[32];
+    double f, h;
+
+    sprintf(buf, "%5.1f", bpm);
+
+    /* Safety catch against bad BPM values, NaN, infinity etc. */
+
+    if (bpm < min || bpm > max) {
+        draw_token(surface, rect, buf, detail_col, bg_col, bg_col);
+        return;
+    }
+
+    /* Colour compatible BPMs the same; cycle 360 degrees
+     * every time the BPM doubles */
+
+    f = log2(bpm);
+    f -= floor(f);
+    h = f * 360.0; /* degrees */
+
+    draw_token(surface, rect, buf, text_col, hsv(h, 1.0, 0.3), bg_col);
+}
+
+/*
+ * Draw the BPM field, or a gap
+ */
+
+static void draw_bpm_field(SDL_Surface *surface, const struct rect *rect,
+                           double bpm, SDL_Color bg_col)
+{
+    if (show_bpm(bpm))
+        draw_bpm(surface, rect, bpm, bg_col);
+    else
+        draw_rect(surface, rect, bg_col);
+}
+
 /*
  * Draw the record information in the deck
  */
@@ -489,14 +661,23 @@ static void draw_rect(SDL_Surface *surface, const struct rect *rect,
 static void draw_record(SDL_Surface *surface, const struct rect *rect,
                         const struct record *record)
 {
-    struct rect top, bottom;
+    struct rect artist, title, left, right;
 
-    split_top(rect, &top, &bottom, FONT_SPACE, 0);
+    split_top(rect, &artist, &title, BIG_FONT_SPACE, 0);
+    draw_text(surface, &artist, record->artist,
+              big_font, text_col, background_col);
 
-    draw_text(surface, &top, record->artist,
-              font, text_col, background_col);
-    draw_text(surface, &bottom, record->title,
-              em_font, text_col, background_col);
+    /* Layout changes slightly if BPM is known */
+
+    if (show_bpm(record->bpm)) {
+        split_left(&title, &left, &right, BPM_WIDTH, 0);
+        draw_bpm(surface, &left, record->bpm, background_col);
+
+        split_left(&right, &left, &title, HALF_SPACER, 0);
+        draw_rect(surface, &left, background_col);
+    }
+
+    draw_text(surface, &title, record->title, font, text_col, background_col);
 }
 
 /*
@@ -644,11 +825,8 @@ static void draw_deck_clocks(SDL_Surface *surface, const struct rect *rect,
     else
         col = text_col;
 
-    if (track_is_importing(track)) {
-        col.r >>= 2;
-        col.g >>= 2;
-        col.b >>= 2;
-    }
+    if (track_is_importing(track))
+        col = dim(col, 2);
 
     draw_clock(surface, &lower, -remain, col);
 }
@@ -707,17 +885,11 @@ static void draw_overview(SDL_Surface *surface, const struct rect *rect,
             fade = 3;
         }
 
-        if (track_is_importing(tr)) {
-            col.b >>= 1;
-            col.g >>= 1;
-            col.r >>= 1;
-        }
+        if (track_is_importing(tr))
+            col = dim(col, 1);
 
-        if (c < current_position) {
-            col.b >>= 1;
-            col.g >>= 1;
-            col.r >>= 1;
-        }
+        if (c < current_position)
+            col = dim(col, 1);
 
         /* Store a pointer to this column of the framebuffer */
 
@@ -918,7 +1090,7 @@ static void draw_deck(SDL_Surface *surface, const struct rect *rect,
 
     position = player_get_elapsed(pl) * t->rate;
 
-    split_top(rect, &track, &rest, FONT_SPACE * 2, 0);
+    split_top(rect, &track, &rest, FONT_SPACE + BIG_FONT_SPACE, 0);
     if (rest.h < 160)
         rest = *rect;
     else
@@ -964,10 +1136,12 @@ static void draw_decks(SDL_Surface *surface, const struct rect *rect,
  * Draw the status bar
  */
 
-static void draw_status(SDL_Surface *sf, const struct rect *rect,
-                        const char *text)
+static void draw_status(SDL_Surface *sf, const struct rect *rect)
 {
-    draw_text(sf, rect, text, detail_font, detail_col, background_col);
+    if (status_level() >= STATUS_ERROR)
+        draw_text(sf, rect, status(), detail_font, text_col, dim(warn_col, 2));
+    else
+        draw_text(sf, rect, status(), detail_font, detail_col, background_col);
 }
 
 /*
@@ -1023,10 +1197,7 @@ static void draw_scroll_bar(SDL_Surface *surface, const struct rect *rect,
     SDL_Rect box;
     SDL_Color bg;
 
-    bg = selected_col;
-    bg.r >>= 1;
-    bg.g >>= 1;
-    bg.b >>= 1;
+    bg = dim(selected_col, 1);
 
     box.x = rect->x;
     box.y = rect->y;
@@ -1051,7 +1222,8 @@ static void draw_scroll_bar(SDL_Surface *surface, const struct rect *rect,
 
 static void draw_crates(SDL_Surface *surface, const struct rect *rect,
                         const struct library *library,
-                        const struct scroll *scroll)
+                        const struct scroll *scroll,
+                        int sort)
 {
     size_t n;
     struct rect left, bottom;
@@ -1062,7 +1234,8 @@ static void draw_crates(SDL_Surface *surface, const struct rect *rect,
     n = scroll->offset;
 
     for (;;) {
-        SDL_Color col_text, col_bg;
+        bool selected;
+        SDL_Color col;
         struct rect top;
         const struct crate *crate;
 
@@ -1075,11 +1248,35 @@ static void draw_crates(SDL_Surface *surface, const struct rect *rect,
         split_top(&bottom, &top, &bottom, FONT_SPACE, 0);
 
         crate = library->crate[n];
+        selected = (n == scroll->selected);
 
-        col_text = crate->is_fixed ? detail_col: text_col;
-        col_bg = (n == scroll->selected) ? selected_col : background_col;
+        col = crate->is_fixed ? detail_col: text_col;
 
-        draw_text(surface, &top, crate->name, font, col_text, col_bg);
+        if (!selected) {
+            draw_text(surface, &top, crate->name, font, col, background_col);
+        } else {
+            struct rect left, right;
+
+            split_right(&top, &left, &right, SORT_WIDTH, 0);
+            draw_text(surface, &left, crate->name, font, col, selected_col);
+
+            switch (sort) {
+            case SORT_ARTIST:
+                draw_token(surface, &right, "ART", text_col,
+                           artist_col, selected_col);
+                break;
+            case SORT_BPM:
+                draw_token(surface, &right, "BPM", text_col,
+                           bpm_col, selected_col);
+                break;
+            case SORT_PLAYLIST:
+                draw_token(surface, &right, "PLS", text_col,
+                           selected_col, selected_col);
+                break;
+            default:
+                abort();
+            }
+        }
 
         n++;
     }
@@ -1087,6 +1284,8 @@ static void draw_crates(SDL_Surface *surface, const struct rect *rect,
     draw_rect(surface, &bottom, background_col);
 }
 
+
+
 /*
  * Display a record library listing, with scrollbar and current
  * selection
@@ -1099,17 +1298,25 @@ static void draw_listing(SDL_Surface *surface, const struct rect *rect,
                          const struct scroll *scroll)
 {
     size_t n;
+    int width;
     struct rect left, bottom;
 
     split_left(rect, &left, &bottom, SCROLLBAR_SIZE, SPACER);
     draw_scroll_bar(surface, &left, scroll);
 
+    /* Choose the width of the 'artist' column */
+
+    width = bottom.w / 2;
+    if (width > RESULTS_ARTIST_WIDTH)
+        width = RESULTS_ARTIST_WIDTH;
+
     n = scroll->offset;
 
     for (;;) {
         SDL_Color col;
         struct rect top, left, right;
         const struct record *record;
+        bool selected;
 
         if (n >= listing->entries)
             break;
@@ -1120,14 +1327,21 @@ static void draw_listing(SDL_Surface *surface, const struct rect *rect,
         split_top(&bottom, &top, &bottom, FONT_SPACE, 0);
 
         record = listing->record[n];
+        selected = (n == scroll->selected);
 
-        if (n == scroll->selected) {
+        if (selected) {
             col = selected_col;
         } else {
             col = background_col;
         }
 
-        split_left(&top, &left, &right, RESULTS_ARTIST_WIDTH, 0);
+        split_left(&top, &left, &right, BPM_WIDTH, 0);
+        draw_bpm_field(surface, &left, record->bpm, col);
+
+        split_left(&right, &left, &right, SPACER, 0);
+        draw_rect(surface, &left, col);
+
+        split_left(&right, &left, &right, width, 0);
         draw_text(surface, &left, record->artist, font, text_col, col);
 
         split_left(&right, &left, &right, SPACER, 0);
@@ -1160,7 +1374,7 @@ static void draw_library(SDL_Surface *surface, const struct rect *rect,
     split_left(&rlists, &rcrates, &rrecords, (rlists.w / 4), SPACER);
     if (rcrates.w > LIBRARY_MIN_WIDTH) {
         draw_listing(surface, &rrecords, sel->view_listing, &sel->records);
-        draw_crates(surface, &rcrates, sel->library, &sel->crates);
+        draw_crates(surface, &rcrates, sel->library, &sel->crates, sel->sort);
     } else {
         draw_listing(surface, rect, sel->view_listing, &sel->records);
     }
@@ -1172,10 +1386,10 @@ static void draw_library(SDL_Surface *surface, const struct rect *rect,
  * Return: true if the selector needs to be redrawn, otherwise false
  */
 
-static bool handle_key(struct deck deck[], size_t ndeck,
-                       struct selector *sel, int *meter_scale,
-                       SDLKey key, SDLMod mod)
+static bool handle_key(SDLKey key, SDLMod mod)
 {
+    struct selector *sel = &selector;
+
     if (key >= SDLK_a && key <= SDLK_z) {
         selector_search_refine(sel, (key - SDLK_a) + 'a');
         return true;
@@ -1229,24 +1443,28 @@ static bool handle_key(struct deck deck[], size_t ndeck,
         return true;
 
     } else if (key == SDLK_TAB) {
-        selector_toggle(sel);
+        if (mod & KMOD_CTRL) {
+            selector_toggle_order(sel);
+        } else {
+            selector_toggle(sel);
+        }
         return true;
 
     } else if ((key == SDLK_EQUALS) || (key == SDLK_PLUS)) {
-        (*meter_scale)--;
+        meter_scale--;
 
-        if (*meter_scale < 0)
-            *meter_scale = 0;
+        if (meter_scale < 0)
+            meter_scale = 0;
 
-        fprintf(stderr, "Meter scale decreased to %d\n", *meter_scale);
+        fprintf(stderr, "Meter scale decreased to %d\n", meter_scale);
 
     } else if (key == SDLK_MINUS) {
-        (*meter_scale)++;
+        meter_scale++;
 
-        if (*meter_scale > MAX_METER_SCALE)
-            *meter_scale = MAX_METER_SCALE;
+        if (meter_scale > MAX_METER_SCALE)
+            meter_scale = MAX_METER_SCALE;
 
-        fprintf(stderr, "Meter scale increased to %d\n", *meter_scale);
+        fprintf(stderr, "Meter scale increased to %d\n", meter_scale);
 
     } else if (key >= SDLK_F1 && key <= SDLK_F12) {
         size_t d;
@@ -1340,14 +1558,26 @@ static Uint32 ticker(Uint32 interval, void *p)
     return interval;
 }
 
+/*
+ * Callback to tell the interface that status has changed
+ */
+
+static void status_change(void)
+{
+    SDL_Event e;
+
+    e.type = SDL_USEREVENT;
+    e.user.code = EVENT_STATUS;
+    SDL_PushEvent(&e);
+}
+
 /*
  * The SDL interface thread
  */
 
 static int interface_main(void)
 {
-    int meter_scale, library_update, decks_update, status_update;
-    const char *status = banner;
+    bool library_update, decks_update, status_update;
 
     SDL_Event event;
     SDL_TimerID timer;
@@ -1355,21 +1585,28 @@ static int interface_main(void)
 
     struct rect rworkspace, rplayers, rlibrary, rstatus, rtmp;
 
-    meter_scale = DEFAULT_METER_SCALE;
-
-    surface = set_size(DEFAULT_WIDTH, DEFAULT_HEIGHT, &rworkspace);
+    surface = set_size(width, height, &rworkspace);
     if (!surface)
         return -1;
 
-    decks_update = UPDATE_REDRAW;
-    status_update = UPDATE_REDRAW;
-    library_update = UPDATE_REDRAW;
+    decks_update = true;
+    status_update = true;
+    library_update = true;
 
     /* The final action is to add the timer which triggers refresh */
 
     timer = SDL_AddTimer(REFRESH, ticker, NULL);
 
-    while (SDL_WaitEvent(&event) >= 0) {
+    rig_lock();
+
+    for (;;) {
+
+        rig_unlock();
+
+        if (SDL_WaitEvent(&event) < 0)
+            break;
+
+        rig_lock();
 
         switch(event.type) {
         case SDL_QUIT: /* user request to quit application; eg. window close */
@@ -1382,31 +1619,43 @@ static int interface_main(void)
             if (!surface)
                 return -1;
 
-            library_update = UPDATE_REDRAW;
-            decks_update = UPDATE_REDRAW;
-            status_update = UPDATE_REDRAW;
+            library_update = true;
+            decks_update = true;
+            status_update = true;
 
             break;
 
         case SDL_USEREVENT:
             switch (event.user.code) {
             case EVENT_TICKER: /* request to poll the clocks */
-                decks_update = UPDATE_REDRAW;
+                decks_update = true;
                 break;
 
             case EVENT_QUIT: /* internal request to finish this thread */
                 goto finish;
 
+            case EVENT_STATUS:
+                status_update = true;
+                break;
+
             default:
                 abort();
             }
             break;
 
         case SDL_KEYDOWN:
-            if (handle_key(deck, ndeck, &selector, &meter_scale,
-                           event.key.keysym.sym, event.key.keysym.mod))
+            if (handle_key(event.key.keysym.sym, event.key.keysym.mod))
             {
-                library_update = UPDATE_REDRAW;
+                struct record *r;
+
+                r = selector_current(&selector);
+                if (r != NULL) {
+                    status_set(STATUS_VERBOSE, r->pathname);
+                } else {
+                    status_set(STATUS_VERBOSE, "No search results found");
+                }
+
+                library_update = true;
             }
 
         } /* switch(event.type) */
@@ -1417,60 +1666,51 @@ static int interface_main(void)
         split_bottom(&rworkspace, &rtmp, &rstatus, STATUS_HEIGHT, SPACER);
         if (rtmp.h < 128 || rtmp.w < 0) {
             rtmp = rworkspace;
-            status_update = UPDATE_NONE;
+            status_update = false;
         }
 
         split_top(&rtmp, &rplayers, &rlibrary, PLAYER_HEIGHT, SPACER);
         if (rlibrary.h < LIBRARY_MIN_HEIGHT || rlibrary.w < LIBRARY_MIN_WIDTH) {
             rplayers = rtmp;
-            library_update = UPDATE_NONE;
+            library_update = false;
         }
 
         if (rplayers.h < 0 || rplayers.w < 0)
-            decks_update = UPDATE_NONE;
+            decks_update = false;
 
-        /* If there's been a change to the library search results,
-         * check them over and display them. */
+        LOCK(surface);
 
-        if (library_update == UPDATE_REDRAW) {
-
-            if (selector_current(&selector) != NULL)
-                status = selector_current(&selector)->pathname;
-            else
-                status = "No search results found";
-            status_update = UPDATE_REDRAW;
-
-            LOCK(surface);
+        if (library_update)
             draw_library(surface, &rlibrary, &selector);
-            UNLOCK(surface);
-            UPDATE(surface, &rlibrary);
-            library_update = UPDATE_NONE;
-        }
 
-        /* If there's been a change to status, redisplay it */
+        if (status_update)
+            draw_status(surface, &rstatus);
 
-        if (status_update == UPDATE_REDRAW) {
-            LOCK(surface);
-            draw_status(surface, &rstatus, status);
-            UNLOCK(surface);
-            UPDATE(surface, &rstatus);
-            status_update = UPDATE_NONE;
-        }
-
-        /* If it's due, redraw the players. This is triggered by the
-         * timer event. */
-
-        if (decks_update == UPDATE_REDRAW) {
-            LOCK(surface);
+        if (decks_update)
             draw_decks(surface, &rplayers, deck, ndeck, meter_scale);
-            UNLOCK(surface);
+
+        UNLOCK(surface);
+
+        if (library_update) {
+            UPDATE(surface, &rlibrary);
+            library_update = false;
+        }
+
+        if (status_update) {
+            UPDATE(surface, &rstatus);
+            status_update = false;
+        }
+
+        if (decks_update) {
             UPDATE(surface, &rplayers);
-            decks_update = UPDATE_NONE;
+            decks_update = false;
         }
 
     } /* main loop */
 
  finish:
+    rig_unlock();
+
     SDL_RemoveTimer(timer);
 
     return 0;
@@ -1482,6 +1722,44 @@ static void* launch(void *p)
     return NULL;
 }
 
+/*
+ * Parse and action the given geometry string. Format is "960x720" or
+ * "970x720+20+20"
+ *
+ * Return: -1 if string could not be actioned, otherwise 0
+ */
+
+static int parse_geometry(const char *s)
+{
+    int n, x, y;
+    char buf[128];
+
+    n = sscanf(s, "%dx%d+%d+%d", &width, &height, &x, &y);
+
+    switch (n) {
+    case EOF: /* empty string */
+        break;
+
+    case 2:
+        break;
+
+    case 4:
+        /* FIXME: Not a desirable way to get geometry information to
+         * SDL, but it seems to be the only way */
+
+        sprintf(buf, "SDL_VIDEO_WINDOW_POS=%d,%d", x, y);
+        if (putenv(buf) != 0)
+            return -1;
+
+        break;
+
+    default:
+        return -1;
+    }
+
+    return 0;
+}
+
 /*
  * Start the SDL interface
  *
@@ -1489,12 +1767,14 @@ static void* launch(void *p)
  * error
  */
 
-int interface_start(struct deck ldeck[], size_t lndeck, struct library *lib)
+int interface_start(struct library *lib, const char *geo)
 {
     size_t n;
 
-    deck = ldeck;
-    ndeck = lndeck;
+    if (parse_geometry(geo) == -1) {
+        fprintf(stderr, "Window geometry ('%s') is not valid.\n", geo);
+        return -1;
+    }
 
     for (n = 0; n < ndeck; n++) {
         if (timecoder_monitor_init(&deck[n].timecoder, SCOPE_SIZE) == -1)
@@ -1503,6 +1783,8 @@ int interface_start(struct deck ldeck[], size_t lndeck, struct library *lib)
 
     selector_init(&selector, lib);
     calculate_spinner_lookup(spinner_angle, NULL, SPINNER_SIZE);
+    status_notify(status_change);
+    status_set(STATUS_VERBOSE, banner);
 
     fprintf(stderr, "Initialising SDL...\n");
 
