@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@xwax.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,7 +23,7 @@
 #include "deck.h"
 #include "library.h"
 
-int interface_start(struct deck deck[], size_t ndeck, struct library *lib);
+int interface_start(struct library *lib, const char *geo);
 void interface_stop();
 
 #endif
