@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@xwax.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,6 +22,6 @@
 
 #include "device.h"
 
-int jack_init(struct device_t *dv, const char *name);
+int jack_init(struct device *dv, const char *name);
 
 #endif
