@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@xwax.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,7 +19,9 @@
 
 #define _GNU_SOURCE /* getdelim(), strdupa() */
 #include <assert.h>
+#include <errno.h>
 #include <libgen.h> /*  basename() */
+#include <math.h> /* isfinite() */
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -50,7 +52,9 @@ static int crate_init(struct crate *c, const char *name, bool is_fixed)
     }
 
     c->is_fixed = is_fixed;
-    listing_init(&c->listing);
+    listing_init(&c->by_artist);
+    listing_init(&c->by_bpm);
+    listing_init(&c->by_order);
 
     return 0;
 }
@@ -64,7 +68,9 @@ static int crate_init(struct crate *c, const char *name, bool is_fixed)
 
 static void crate_clear(struct crate *c)
 {
-    listing_clear(&c->listing);
+    listing_clear(&c->by_artist);
+    listing_clear(&c->by_bpm);
+    listing_clear(&c->by_order);
     free(c->name);
 }
 
@@ -82,6 +88,36 @@ static int crate_cmp(const struct crate *a, const struct crate *b)
     return strcmp(a->name, b->name);
 }
 
+/*
+ * Add a record into a crate and its various indexes
+ *
+ * FIXME: not all out-of-memory cases are implemented
+ *
+ * Return: Pointer to existing entry, NULL if out of memory
+ * Post: Record added to the crate
+ */
+
+static struct record* crate_add(struct crate *c, struct record *r)
+{
+    struct record *x;
+
+    assert(r != NULL);
+
+    x = listing_insert(&c->by_artist, r, SORT_ARTIST);
+    if (x != r) /* may be NULL */
+        return x;
+
+    x = listing_insert(&c->by_bpm, r, SORT_BPM);
+    if (x == NULL)
+        abort(); /* FIXME: remove from all listings and return */
+    assert(x == r);
+
+    if (listing_add(&c->by_order, r) != 0)
+        abort(); /* FIXME: remove from all listings and return */
+
+    return r;
+}
+
 /*
  * Comparison function, see qsort(3)
  */
@@ -151,7 +187,7 @@ struct crate* get_crate(struct library *lib, const char *name)
  * Return: pointer to crate, or NULL on memory allocation failure
  */
 
-struct crate* use_crate(struct library *lib, char *name, bool is_fixed)
+struct crate* use_crate(struct library *lib, char *name)
 {
     struct crate *new_crate;
 
@@ -169,7 +205,7 @@ struct crate* use_crate(struct library *lib, char *name, bool is_fixed)
         return NULL;
     }
 
-    if (crate_init(new_crate, name, is_fixed) == -1)
+    if (crate_init(new_crate, name, false) == -1)
         goto fail;
 
     if (add_crate(lib, new_crate) == -1)
@@ -225,10 +261,10 @@ void library_clear(struct library *li)
 
     /* This object is responsible for all the record pointers */
 
-    for (n = 0; n < li->all.listing.entries; n++) {
+    for (n = 0; n < li->all.by_artist.entries; n++) {
         struct record *re;
 
-        re = li->all.listing.record[n];
+        re = li->all.by_artist.record[n];
         record_clear(re);
         free(re);
     }
@@ -295,6 +331,28 @@ done:
     return s;
 }
 
+/*
+ * Translate a string from the scan to our internal BPM value
+ *
+ * Return: internal BPM value, or INFINITY if string is malformed
+ */
+
+static double parse_bpm(const char *s)
+{
+    char *endptr;
+    double bpm;
+
+    if (s[0] == '\0') /* empty string, valid for 'unknown BPM' */
+        return 0.0;
+
+    errno = 0;
+    bpm = strtod(s, &endptr);
+    if (errno == ERANGE || *endptr != '\0' || !isfinite(bpm) || bpm <= 0.0)
+        return INFINITY;
+
+    return bpm;
+}
+
 /*
  * Read the next record from the file
  *
@@ -306,11 +364,12 @@ done:
 static int get_record(FILE *f, struct record **r)
 {
     struct record x, *y;
-    char delim;
+    char delim, *s;
 
     x.pathname = NULL;
     x.artist = NULL;
     x.title = NULL;
+    x.bpm = 0.0;
 
     x.pathname = get_field(f, &delim);
     if (x.pathname == NULL)
@@ -342,11 +401,34 @@ static int get_record(FILE *f, struct record **r)
     if (x.title == NULL)
         goto fail;
 
+    if (delim == '\n') /* other fields are optional */
+        goto done;
+
+    if (delim != '\t') {
+        fprintf(stderr, "Malformed record '%s'\n", x.pathname);
+        goto fail;
+    }
+
+    /* Beats-per-minute (BPM) */
+
+    s = get_field(f, &delim);
+    if (s == NULL)
+        goto fail;
+
+    x.bpm = parse_bpm(s);
+    if (!isfinite(x.bpm)) {
+        fprintf(stderr, "%s: Ignoring malformed BPM '%s'\n", x.pathname, s);
+        x.bpm = 0.0;
+    }
+
+    free(s);
+
     if (delim != '\n') {
         fprintf(stderr, "Malformed record '%s'\n", x.pathname);
         goto fail;
     }
 
+done:
     y = malloc(sizeof *y);
     if (y == NULL) {
         perror("malloc");
@@ -375,24 +457,20 @@ fail:
  * Return: 0 on success, -1 on fatal error (may leak)
  */
 
-int library_import(struct library *li, bool sort,
-                   const char *scan, const char *path)
+int library_import(struct library *li, const char *scan, const char *path)
 {
     int fd, status;
     char *cratename, *pathname;
     pid_t pid;
     FILE *fp;
-    struct crate *crate, *all_crate;
+    struct crate *crate;
 
     fprintf(stderr, "Scanning '%s'...\n", path);
 
-    all_crate = get_crate(li, CRATE_ALL);
-    assert(all_crate != NULL);
-
     pathname = strdupa(path);
     cratename = basename(pathname); /* POSIX version, see basename(3) */
     assert(cratename != NULL);
-    crate = use_crate(li, cratename, false);
+    crate = use_crate(li, cratename);
     if (crate == NULL)
         return -1;
 
@@ -417,7 +495,7 @@ int library_import(struct library *li, bool sort,
 
         /* Add to the crate of all records */
 
-        x = listing_insert(&all_crate->listing, d);
+        x = crate_add(&li->all, d);
         if (x == NULL)
             return -1;
 
@@ -431,13 +509,8 @@ int library_import(struct library *li, bool sort,
 
         /* Insert into the user's crate */
 
-        if (sort) {
-            if (listing_insert(&crate->listing, d) == NULL)
-                return -1;
-        } else {
-            if (listing_add(&crate->listing, d) == -1)
-                return -1;
-        }
+        if (crate_add(crate, d) == NULL)
+            return -1;
     }
 
     if (fclose(fp) == -1) {
