@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@xwax.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,9 +28,9 @@
 /* A single crate of records */
 
 struct crate {
-    bool is_fixed;
+    bool is_fixed, is_ordered;
     char *name;
-    struct listing listing;
+    struct listing by_artist, by_bpm, by_order;
 };
 
 /* The complete music library, which consists of multiple crates */
@@ -43,7 +43,6 @@ struct library {
 int library_init(struct library *li);
 void library_clear(struct library *li);
 
-int library_import(struct library *lib, bool sort,
-                   const char *scan, const char *path);
+int library_import(struct library *lib, const char *scan, const char *path);
 
 #endif
