@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@xwax.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,8 +22,14 @@
 
 #include <stddef.h>
 
+#define SORT_ARTIST   0
+#define SORT_BPM      1
+#define SORT_PLAYLIST 2
+#define SORT_END      3
+
 struct record {
     char *pathname, *artist, *title;
+    double bpm; /* or 0.0 if not known */
 };
 
 /* Listing points to records, but does not manage those pointers */
@@ -40,8 +46,9 @@ int listing_add(struct listing *li, struct record *lr);
 int listing_copy(const struct listing *src, struct listing *dest);
 int listing_match(struct listing *src, struct listing *dest,
 		  const char *match);
-struct record* listing_insert(struct listing *ls, struct record *item);
+struct record* listing_insert(struct listing *ls, struct record *item,
+                              int sort);
+size_t listing_find(struct listing *ls, struct record *item, int sort);
 void listing_debug(struct listing *ls);
-void listing_sort(struct listing *ls);
 
 #endif
