@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@xwax.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,13 +27,13 @@
 
 /* State of the pitch calculation filter */
 
-struct pitch_t {
+struct pitch {
     double dt, x, v;
 };
 
 /* Prepare the filter for observations every dt seconds */
 
-static inline void pitch_init(struct pitch_t *p, double dt)
+static inline void pitch_init(struct pitch *p, double dt)
 {
     p->dt = dt;
     p->x = 0.0;
@@ -46,7 +46,7 @@ static inline void pitch_init(struct pitch_t *p, double dt)
  * Because the vinyl uses timestamps, the values for dx are discrete
  * rather than smooth. */
 
-static inline void pitch_dt_observation(struct pitch_t *p, double dx)
+static inline void pitch_dt_observation(struct pitch *p, double dx)
 {
     double predicted_x, predicted_v, residual_x;
 
@@ -63,7 +63,7 @@ static inline void pitch_dt_observation(struct pitch_t *p, double dx)
 
 /* Get the pitch after filtering */
 
-static inline double pitch_current(struct pitch_t *p)
+static inline double pitch_current(struct pitch *p)
 {
     return p->v;
 }
