@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@xwax.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -400,6 +400,15 @@ void retarget(struct player *pl)
     }
 }
 
+/*
+ * Seek to the given position
+ */
+
+void player_seek_to(struct player *pl, double seconds)
+{
+    pl->offset = pl->position - seconds;
+}
+
 /*
  * Get a block of PCM audio data to send to the soundcard
  *
