@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@xwax.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,54 +17,16 @@
  *
  */
 
+#include <assert.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 
+#include "controller.h"
+#include "debug.h"
 #include "device.h"
 #include "realtime.h"
-
-#define REALTIME_PRIORITY 80
-
-static pthread_key_t key;
-
-/*
- * Put in place checks for realtime and non-realtime threads
- */
-
-int rt_global_init(void)
-{
-    int r;
-
-    r = pthread_key_create(&key, NULL);
-    if (r != 0) {
-        errno = r;
-        perror("pthread_key_create");
-        return -1;
-    }
-
-    if (pthread_setspecific(key, (void*)false) != 0)
-        abort();
-
-    return 0;
-}
-
-/*
- * Check for programmer error
- *
- * Pre: the current thread is non realtime
- */
-
-void rt_not_allowed()
-{
-    bool rt;
-
-    rt = (bool)pthread_getspecific(key);
-    if (rt) {
-        fprintf(stderr, "Realtime thread called a blocking function\n");
-        abort();
-    }
-}
+#include "thread.h"
 
 /*
  * Raise the priority of the current thread
@@ -72,23 +34,24 @@ void rt_not_allowed()
  * Return: -1 if priority could not be satisfactorily raised, otherwise 0
  */
 
-static int raise_priority()
+static int raise_priority(int priority)
 {
     int max_pri;
     struct sched_param sp;
 
+    max_pri = sched_get_priority_max(SCHED_FIFO);
+
+    if (priority > max_pri) {
+        fprintf(stderr, "Invalid scheduling priority (maximum %d).\n", max_pri);
+        return -1;
+    }
+
     if (sched_getparam(0, &sp)) {
         perror("sched_getparam");
         return -1;
     }
 
-    max_pri = sched_get_priority_max(SCHED_FIFO);
-    sp.sched_priority = REALTIME_PRIORITY;
-
-    if (sp.sched_priority > max_pri) {
-        fprintf(stderr, "Invalid scheduling priority (maximum %d).\n", max_pri);
-        return -1;
-    }
+    sp.sched_priority = priority;
 
     if (sched_setscheduler(0, SCHED_FIFO, &sp)) {
         perror("sched_setscheduler");
@@ -96,12 +59,6 @@ static int raise_priority()
         return -1;
     }
 
-    /* Set thread local storage to show that this thread is
-     * realtime, for assertions later */
-
-    if (pthread_setspecific(key, (void*)true) != 0)
-        abort();
-
     return 0;
 }
 
@@ -114,8 +71,14 @@ static void rt_main(struct rt *rt)
     int r;
     size_t n;
 
-    if (raise_priority() == -1)
-        rt->finished = true;
+    debug("%p", rt);
+
+    thread_to_realtime();
+
+    if (rt->priority != 0) {
+        if (raise_priority(rt->priority) == -1)
+            rt->finished = true;
+    }
 
     if (sem_post(&rt->sem) == -1)
         abort(); /* under our control; see sem_post(3) */
@@ -131,6 +94,9 @@ static void rt_main(struct rt *rt)
             }
         }
 
+        for (n = 0; n < rt->nctl; n++)
+            controller_handle(rt->ctl[n]);
+
         for (n = 0; n < rt->ndv; n++)
             device_handle(rt->dv[n]);
     }
@@ -148,8 +114,11 @@ static void* launch(void *p)
 
 void rt_init(struct rt *rt)
 {
+    debug("%p", rt);
+
     rt->finished = false;
     rt->ndv = 0;
+    rt->nctl = 0;
     rt->npt = 0;
 }
 
@@ -172,6 +141,8 @@ int rt_add_device(struct rt *rt, struct device *dv)
 {
     ssize_t z;
 
+    debug("%p adding device %p", rt, dv);
+
     if (rt->ndv == sizeof rt->dv) {
         fprintf(stderr, "Too many audio devices\n");
         return -1;
@@ -194,6 +165,29 @@ int rt_add_device(struct rt *rt, struct device *dv)
     return 0;
 }
 
+/*
+ * Add a controller to the realtime handler
+ *
+ * Return: -1 if the device could not be added, otherwise 0
+ */
+
+int rt_add_controller(struct rt *rt, struct controller *c)
+{
+    debug("%p adding controller %p", rt, c);
+
+    if (rt->nctl == sizeof rt->ctl) {
+        fprintf(stderr, "Too many controllers\n");
+        return -1;
+    }
+
+    /* Controllers don't have poll entries; they are polled every
+     * cycle of the audio */
+
+    rt->ctl[rt->nctl++] = c;
+
+    return 0;
+}
+
 /*
  * Start realtime handling of the given devices
  *
@@ -203,10 +197,13 @@ int rt_add_device(struct rt *rt, struct device *dv)
  * Return: -1 on error, otherwise 0
  */
 
-int rt_start(struct rt *rt)
+int rt_start(struct rt *rt, int priority)
 {
     size_t n;
 
+    assert(priority >= 0);
+    rt->priority = priority;
+
     /* If there are any devices which returned file descriptors for
      * poll() then launch the realtime thread to handle them */
 
