@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@xwax.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -34,10 +34,14 @@ struct rt {
     pthread_t ph;
     sem_t sem;
     bool finished;
+    int priority;
 
     size_t ndv;
     struct device *dv[3];
 
+    size_t nctl;
+    struct controller *ctl[3];
+
     size_t npt;
     struct pollfd pt[32];
 };
@@ -49,8 +53,9 @@ void rt_init(struct rt *rt);
 void rt_clear(struct rt *rt);
 
 int rt_add_device(struct rt *rt, struct device *dv);
+int rt_add_controller(struct rt *rt, struct controller *c);
 
-int rt_start(struct rt *rt);
+int rt_start(struct rt *rt, int priority);
 void rt_stop(struct rt *rt);
 
 #endif
