@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>,
+ * Copyright (C) 2012 Mark Hills <mark@xwax.org>,
  *                    Yves Adler <yves.adler@googlemail.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -43,7 +43,8 @@ struct selector {
 
     struct scroll records, crates;
     bool toggled;
-    int toggle_back;
+    int toggle_back, sort;
+    struct record *target;
 
     size_t search_len;
     char search[256];
@@ -66,6 +67,7 @@ struct record* selector_current(struct selector *sel);
 void selector_prev(struct selector *sel);
 void selector_next(struct selector *sel);
 void selector_toggle(struct selector *sel);
+void selector_toggle_order(struct selector *sel);
 
 void selector_search_expand(struct selector *sel);
 void selector_search_refine(struct selector *sel, char key);
