@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@xwax.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,6 +20,7 @@
 #include <stdio.h>
 
 #include "rig.h"
+#include "thread.h"
 #include "track.h"
 
 /*
@@ -35,6 +36,9 @@ int main(int argc, char *argv[])
         return -1;
     }
 
+    if (thread_global_init() == -1)
+        return -1;
+
     rig_init();
 
     track = track_get_by_import(argv[1], argv[2]);
@@ -45,6 +49,7 @@ int main(int argc, char *argv[])
 
     track_put(track);
     rig_clear();
+    thread_global_clear();
 
     return 0;
 }
