@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@xwax.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -194,7 +194,7 @@ static inline bits_t rev(bits_t current, struct timecode_def *def)
 static int build_lookup(struct timecode_def *def)
 {
     unsigned int n;
-    bits_t current, last;
+    bits_t current;
 
     if (def->lookup)
         return 0;
@@ -208,12 +208,17 @@ static int build_lookup(struct timecode_def *def)
     current = def->seed;
 
     for (n = 0; n < def->length; n++) {
+        bits_t next;
+
         /* timecode must not wrap */
         dassert(lut_lookup(&def->lut, current) == (unsigned)-1);
         lut_push(&def->lut, current);
-        last = current;
-        current = fwd(current, def);
-        dassert(rev(current, def) == last);
+
+        /* check symmetry of the lfsr functions */
+        next = fwd(current, def);
+        dassert(rev(next, def) == current);
+
+        current = next;
     }
 
     def->lookup = true;
@@ -455,7 +460,7 @@ static void process_bitstream(struct timecoder *tc, signed int m)
 
     tc->ref_level = (tc->ref_level * (REF_PEAKS_AVG - 1) + m) / REF_PEAKS_AVG;
 
-    debug("%+6d zero, %+6d (ref %+6d)\t= %d%c (%5d)\n",
+    debug("%+6d zero, %+6d (ref %+6d)\t= %d%c (%5d)",
           tc->primary.zero,
           m, tc->ref_level,
 	  b, tc->valid_counter == 0 ? 'x' : ' ',
@@ -588,7 +593,7 @@ void timecoder_submit(struct timecoder *tc, signed short *pcm, size_t npcm)
  * read.
  *
  * Return: the known position of the timecode, or -1 if not known
- * Post: if when != -1, *when contains the elapsed time in seconds
+ * Post: if when != NULL, *when is the elapsed time in seconds
  */
 
 signed int timecoder_get_position(struct timecoder *tc, double *when)
