@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@xwax.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,21 +19,32 @@
 
 #define _BSD_SOURCE /* vfork() */
 #include <assert.h>
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <unistd.h>
+#include <sys/types.h>
+#include <sys/wait.h>
 #include <sys/mman.h> /* mlock() */
 
 #include "debug.h"
+#include "external.h"
 #include "list.h"
 #include "realtime.h"
 #include "rig.h"
+#include "status.h"
 #include "track.h"
 
+#define RATE 44100
+
 #define SAMPLE (sizeof(signed short) * TRACK_CHANNELS) /* bytes per sample */
 #define TRACK_BLOCK_PCM_BYTES (TRACK_BLOCK_SAMPLES * SAMPLE)
 
-struct list tracks = LIST_INIT(tracks);
-bool use_mlock = false;
+#define _STR(tok) #tok
+#define STR(tok) _STR(tok)
+
+static struct list tracks = LIST_INIT(tracks);
+static bool use_mlock = false;
 
 /*
  * An empty track is used rarely, and is easier than
@@ -43,12 +54,12 @@ bool use_mlock = false;
 static struct track empty = {
     .refcount = 1,
 
-    .rate = TRACK_RATE,
+    .rate = RATE,
     .bytes = 0,
     .length = 0,
     .blocks = 0,
 
-    .importing = false
+    .pid = 0
 };
 
 /*
@@ -95,7 +106,7 @@ static int more_space(struct track *tr)
 
     tr->block[tr->blocks++] = block;
 
-    debug("Allocated new track block (%d blocks, %zu bytes).\n",
+    debug("allocated new track block (%d blocks, %zu bytes)",
           tr->blocks, tr->blocks * TRACK_BLOCK_SAMPLES * SAMPLE);
 
     return 0;
@@ -108,7 +119,7 @@ static int more_space(struct track *tr)
  * Post: len contains the length of the buffer, in bytes
  */
 
-void* track_access_pcm(struct track *tr, size_t *len)
+static void* access_pcm(struct track *tr, size_t *len)
 {
     unsigned int block;
     size_t fill;
@@ -130,9 +141,6 @@ void* track_access_pcm(struct track *tr, size_t *len)
  *
  * The parameter is the number of stereo samples which have been
  * placed in the buffer.
- *
- * Pre: the number of samples does not overflow the size of the buffer,
- * given by track_access_pcm()
  */
 
 static void commit_pcm_samples(struct track *tr, unsigned int samples)
@@ -191,12 +199,9 @@ static void commit_pcm_samples(struct track *tr, unsigned int samples)
  *
  * This function passes any whole samples to commit_pcm_samples()
  * and leaves the residual in the buffer ready for next time.
- *
- * Pre: len is not greater than the size of the buffer, available
- * from track_access_pcm()
  */
 
-void track_commit(struct track *tr, size_t len)
+static void commit(struct track *tr, size_t len)
 {
     tr->bytes += len;
     commit_pcm_samples(tr, tr->bytes / SAMPLE - tr->length);
@@ -207,30 +212,36 @@ void track_commit(struct track *tr, size_t len)
  * importing the data
  *
  * Post: track is initialised
+ * Post: track is importing
  */
 
-static int track_init(struct track *t, const char *importer,
-                       const char *path)
+static int track_init(struct track *t, const char *importer, const char *path)
 {
+    pid_t pid;
+
+    fprintf(stderr, "Importing '%s'...\n", path);
+
+    pid = fork_pipe_nb(&t->fd, importer, "import", path, STR(RATE), NULL);
+    if (pid == -1)
+        return -1;
+
+    t->pid = pid;
+    t->pe = NULL;
+    t->terminated = false;
+
     t->refcount = 0;
 
     t->blocks = 0;
-    t->rate = TRACK_RATE;
+    t->rate = RATE;
 
     t->bytes = 0;
     t->length = 0;
     t->ppm = 0;
     t->overview = 0;
 
-    t->importing = true;
-    t->has_poll = false;
-
     t->importer = importer;
     t->path = path;
 
-    if (import_start(&t->import, t, importer, path) == -1)
-        return -1;
-
     list_add(&t->tracks, &tracks);
     rig_post_track(t);
 
@@ -243,6 +254,7 @@ static int track_init(struct track *t, const char *importer,
  * Terminates any import processes and frees any memory allocated by
  * this object.
  *
+ * Pre: track is not importing
  * Pre: track is initialised
  */
 
@@ -250,7 +262,7 @@ static void track_clear(struct track *tr)
 {
     int n;
 
-    assert(!tr->importing);
+    assert(tr->pid == 0);
 
     for (n = 0; n < tr->blocks; n++)
         free(tr->block[n]);
@@ -311,7 +323,7 @@ struct track* track_get_by_import(const char *importer, const char *path)
 /*
  * Get a pointer to a static track containing no audio
  *
- * Return: pointer
+ * Return: pointer, not NULL
  */
 
 struct track* track_get_empty(void)
@@ -325,6 +337,20 @@ void track_get(struct track *t)
     t->refcount++;
 }
 
+/*
+ * Request premature termination of an import operation
+ */
+
+static void terminate(struct track *t)
+{
+    assert(t->pid != 0);
+
+    if (kill(t->pid, SIGTERM) == -1)
+        abort();
+
+    t->terminated = true;
+}
+
 /*
  * Finish use of a track object
  */
@@ -336,13 +362,13 @@ void track_put(struct track *t)
     /* When importing, a reference is held. If it's the
      * only one remaining terminate it to save resources */
 
-    if (t->refcount == 1 && t->importing) {
-        import_terminate(&t->import);
+    if (t->refcount == 1 && t->pid != 0) {
+        terminate(t);
         return;
     }
 
-    if (t->refcount == 0 && t != &empty) {
-        assert(!t->importing);
+    if (t->refcount == 0) {
+        assert(t != &empty);
         track_clear(t);
         free(t);
     }
@@ -351,16 +377,85 @@ void track_put(struct track *t)
 /*
  * Get entry for use by poll()
  *
- * Return: number of file descriptors placed in *pe
- * Post: *pe contains 0 or 1 file descriptors
+ * Pre: track is importing
+ * Post: *pe contains poll entry
  */
 
 void track_pollfd(struct track *t, struct pollfd *pe)
 {
-    assert(t->importing);
+    assert(t->pid != 0);
 
-    import_pollfd(&t->import, pe);
-    t->has_poll = true;
+    pe->fd = t->fd;
+    pe->events = POLLIN;
+
+    t->pe = pe;
+}
+
+/*
+ * Read the next block of data from the file handle into the track's
+ * PCM data
+ *
+ * Return: -1 on completion, otherwise zero
+ */
+
+static int read_from_pipe(struct track *tr)
+{
+    for (;;) {
+        void *pcm;
+        size_t len;
+        ssize_t z;
+
+        pcm = access_pcm(tr, &len);
+        if (pcm == NULL)
+            return -1;
+
+        z = read(tr->fd, pcm, len);
+        if (z == -1) {
+            if (errno == EAGAIN) {
+                return 0;
+            } else {
+                perror("read");
+                return -1;
+            }
+        }
+
+        if (z == 0) /* EOF */
+            break;
+
+        commit(tr, z);
+    }
+
+    return -1; /* completion without error */
+}
+
+/*
+ * Synchronise with the import process and complete it
+ *
+ * Pre: track is importing
+ * Post: track is not importing
+ */
+
+static void stop_import(struct track *t)
+{
+    int status;
+
+    assert(t->pid != 0);
+
+    if (close(t->fd) == -1)
+        abort();
+
+    if (waitpid(t->pid, &status, 0) == -1)
+        abort();
+
+    if (WIFEXITED(status) && WEXITSTATUS(status) == EXIT_SUCCESS) {
+        fprintf(stderr, "Track import completed\n");
+    } else {
+        fprintf(stderr, "Track import completed with status %d\n", status);
+        if (!t->terminated)
+            status_printf(STATUS_ERROR, "Error importing %s", t->path);
+    }
+
+    t->pid = 0;
 }
 
 /*
@@ -369,19 +464,23 @@ void track_pollfd(struct track *t, struct pollfd *pe)
  * Return: true if import has completed, otherwise false
  */
 
-bool track_handle(struct track *tr)
+void track_handle(struct track *tr)
 {
+    assert(tr->pid != 0);
+
     /* A track may be added while poll() was waiting,
      * in which case it has no return data from poll */
 
-    if (!tr->has_poll)
-        return false;
+    if (tr->pe == NULL)
+        return;
 
-    if (import_handle(&tr->import) == -1) {
-        import_stop(&tr->import);
-        tr->importing = false;
-        return true;
-    }
+    if (tr->pe->revents == 0)
+        return;
 
-    return false;
+    if (read_from_pipe(tr) != -1)
+        return;
+
+    stop_import(tr);
+    list_del(&tr->rig);
+    track_put(tr); /* may delete the track */
 }
