@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2012 Mark Hills <mark@xwax.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,7 +28,6 @@
 
 #include "alsa.h"
 #include "controller.h"
-#include "deck.h"
 #include "device.h"
 #include "dicer.h"
 #include "interface.h"
@@ -57,7 +56,10 @@
 #define ARRAY_SIZE(x) (sizeof(x) / sizeof(*x))
 
 char *banner = "xwax " VERSION \
-    " (C) Copyright 2012 Mark Hills <mark@pogo.org.uk>";
+    " (C) Copyright 2012 Mark Hills <mark@xwax.org>";
+
+size_t ndeck;
+struct deck deck[3];
 
 static void usage(FILE *fd)
 {
@@ -66,12 +68,12 @@ static void usage(FILE *fd)
     fprintf(fd, "Program-wide options:\n"
       "  -k             Lock real-time memory into RAM\n"
       "  -q <n>         Real-time priority (0 for no priority, default %d)\n"
+      "  -g <n>x<n>     Set display geometry\n"
       "  -h             Display this message to stdout and exit\n\n",
       DEFAULT_PRIORITY);
 
     fprintf(fd, "Music library options:\n"
       "  -l <path>      Location to scan for audio tracks\n"
-      "  -p <path>      Ordered playlist for audio tracks\n"
       "  -s <program>   Library scanner (default '%s')\n\n",
       DEFAULT_SCANNER);
 
@@ -123,15 +125,14 @@ static void usage(FILE *fd)
 
 int main(int argc, char *argv[])
 {
-    int r, n, decks, priority;
-    const char *importer, *scanner;
+    int r, n, priority;
+    const char *importer, *scanner, *geo;
     char *endptr;
     size_t nctl;
     double speed;
     struct timecode_def *timecode;
     bool protect, use_mlock;
 
-    struct deck deck[3];
     struct controller ctl[2];
     struct rt rt;
     struct library library;
@@ -158,7 +159,8 @@ int main(int argc, char *argv[])
     rt_init(&rt);
     library_init(&library);
 
-    decks = 0;
+    ndeck = 0;
+    geo = "";
     nctl = 0;
     priority = DEFAULT_PRIORITY;
     importer = DEFAULT_IMPORTER;
@@ -295,14 +297,14 @@ int main(int argc, char *argv[])
                 return -1;
             }
 
-            if (decks == ARRAY_SIZE(deck)) {
+            if (ndeck == ARRAY_SIZE(deck)) {
                 fprintf(stderr, "Too many decks; aborting.\n");
                 return -1;
             }
 
-            fprintf(stderr, "Initialising deck %d (%s)...\n", decks, argv[1]);
+            fprintf(stderr, "Initialising deck %d (%s)...\n", ndeck, argv[1]);
 
-            ld = &deck[decks];
+            ld = &deck[ndeck];
             device = &ld->device;
             timecoder = &ld->timecoder;
             ld->importer = importer;
@@ -357,9 +359,9 @@ int main(int argc, char *argv[])
             /* Connect this deck to available controllers */
 
             for (n = 0; n < nctl; n++)
-                controller_add_deck(&ctl[n], &deck[decks]);
+                controller_add_deck(&ctl[n], &deck[ndeck]);
 
-            decks++;
+            ndeck++;
 
             argv += 2;
             argc -= 2;
@@ -440,6 +442,18 @@ int main(int argc, char *argv[])
             argv += 2;
             argc -= 2;
 
+        } else if (!strcmp(argv[0], "-g")) {
+
+            if (argc < 2) {
+                fprintf(stderr, "-g requires an argument.\n");
+                return -1;
+            }
+
+            geo = argv[1];
+
+            argv += 2;
+            argc -= 2;
+
         } else if (!strcmp(argv[0], "-i")) {
 
             /* Importer script for subsequent decks */
@@ -470,9 +484,7 @@ int main(int argc, char *argv[])
             argv += 2;
             argc -= 2;
 
-        } else if (!strcmp(argv[0], "-l") || !strcmp(argv[0], "-p")) {
-
-            bool sort;
+        } else if (!strcmp(argv[0], "-l")) {
 
             /* Load in a music library */
 
@@ -482,13 +494,7 @@ int main(int argc, char *argv[])
                 return -1;
             }
 
-            if (argv[0][1] == 'p') {
-                sort = false;
-            } else {
-                sort = true;
-            }
-
-            if (library_import(&library, sort, scanner, argv[1]) == -1)
+            if (library_import(&library, scanner, argv[1]) == -1)
                 return -1;
 
             argv += 2;
@@ -530,7 +536,7 @@ int main(int argc, char *argv[])
     alsa_clear_config_cache();
 #endif
 
-    if (decks == 0) {
+    if (ndeck == 0) {
         fprintf(stderr, "You need to give at least one audio device to use "
                 "as a deck; try -h.\n");
         return -1;
@@ -551,7 +557,7 @@ int main(int argc, char *argv[])
         return -1;
     }
 
-    if (interface_start(deck, decks, &library) == -1)
+    if (interface_start(&library, geo) == -1)
         return -1;
 
     if (rig_main() == -1)
@@ -562,7 +568,7 @@ int main(int argc, char *argv[])
     interface_stop();
     rt_stop(&rt);
 
-    for (n = 0; n < decks; n++)
+    for (n = 0; n < ndeck; n++)
         deck_clear(&deck[n]);
 
     for (n = 0; n < nctl; n++)
